@@ -1,33 +1,6 @@
-/*
- * wreport/var - Store a value and its informations
- *
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
- * Author: Enrico Zini <enrico@enricozini.com>
- */
-
 #ifndef WREPORT_VAR_H
 #define WREPORT_VAR_H
 
-/** @file
- * @ingroup core
- * Implement wreport::Var, an encapsulation of a measured variable.
- */
-
-
 #include <wreport/error.h>
 #include <wreport/varinfo.h>
 #include <cstdio>
@@ -39,13 +12,13 @@ struct lua_State;
 namespace wreport {
 
 /**
- * Holds a wreport variable
+ * A physical variable
  *
  * A wreport::Var contains:
- * \li a wreport::Varcode identifying what is measured.  See @ref vartable.h
- * \li a measured value, that can be an integer, double or string depending on
- *     the wreport::Varcode
- * \li zero or more attributes, in turn represented by wreport::Var structures
+ * \li a wreport::Varinfo describing the variable
+ * \li a value, that can be integer, floating point, string or opaque binary
+ *     data as specified by the Varinfo
+ * \li zero or more attributes, represented by other wreport::Var objects
  */
 class Var
 {
@@ -53,12 +26,40 @@ protected:
 	/// Metadata about the variable
 	Varinfo m_info;
 
-	/// Value of the variable
-	char* m_value;
+    /// True if the variable is set, false otherwise
+    bool m_isset;
+
+    /**
+     * Value of the variable
+     *
+     * For numeric values, it is the value encoded to an integer decimal string
+     * according to m_info.
+     *
+     * For string values, it is the 0-terminated string.
+     *
+     * For binary values, it is a raw buffer where the first m_info->bit_len
+     * bits are the binary value, and the rest is set to 0.
+     */
+    union {
+        int32_t i;
+        char* c;
+    } m_value;
 
 	/// Attribute list (ordered by Varcode)
 	Var* m_attrs;
 
+    /// Make sure that m_value is allocated. It does nothing if it already is.
+    void allocate();
+
+    /// Copy the value from var. var is assumed to have the same varinfo as us.
+    void copy_value(const Var& var);
+    /// Move the value from var. var is assumed to have the same varinfo as us. var is left unset.
+    void move_value(Var& var);
+    void assign_i_checked(int32_t val);
+    void assign_d_checked(double val);
+    void assign_b_checked(uint8_t* val, unsigned size);
+    void assign_c_checked(const char* val, unsigned size);
+
 public:
 	/// Create a new Var, with undefined value
 	Var(Varinfo info);
@@ -72,34 +73,47 @@ public:
 	/// Create a new Var, with character value
 	Var(Varinfo info, const char* val);
 
+    /// Create a new Var, with character value
+    Var(Varinfo info, const std::string& val);
+
+    /**
+     * Create a new Var with the value from another one.
+     *
+     * Conversions are applied if necessary, attributes are not copied.
+     *
+     * @param info
+     *   The wreport::Varinfo describing the variable to create
+     * @param var
+     *   The variable with the value to use
+     */
+    Var(Varinfo info, const Var& var);
+
 	/// Copy constructor
 	Var(const Var& var);
 
-    /// Copy constructor
-    Var(const Var& var, bool with_attrs);
+    /**
+     * Move constructor.
+     *
+     * After movement, \a var will still a valid variable, but it will be unset
+     * and without attributes.
+     */
+    Var(Var&& var);
 
-	/**
-	 * Create a new Var with the value from another one
-	 *
-	 * Conversions are applied if necessary
-	 *
-	 * @param info
-	 *   The wreport::Varinfo describing the variable to create
-	 * @param var
-	 *   The variable with the value to use
-	 */
-	Var(Varinfo info, const Var& var);
+    ~Var();
 
-	~Var();
-	
-	/// Assignment
-	Var& operator=(const Var& var);
+    /// Assignment
+    Var& operator=(const Var& var);
 
-	/// Equality
-	bool operator==(const Var& var) const;
+    /**
+     * Move assignment
+     *
+     * After movement, \a var will still a valid variable, but it will be unset
+     * and without attributes.
+     */
+    Var& operator=(Var&& var);
 
-	/// Equality
-	bool operator!=(const Var& var) const { return !operator==(var); }
+    bool operator==(const Var& var) const;
+    bool operator!=(const Var& var) const { return !operator==(var); }
 
     /**
      * Test if the values are the same, regardless of variable codes or
@@ -107,17 +121,15 @@ public:
      */
     bool value_equals(const Var& var) const;
 
-	/// Retrieve the Varcode for a variable
-	Varcode code() const throw ();
+    /// Retrieve the Varcode for a variable
+    Varcode code() const throw () { return m_info->code; }
 
-	/// Get informations about the variable
-	Varinfo info() const throw ();
+    /// Get informations about the variable
+    Varinfo info() const throw () { return m_info; }
 
-	/// Retrieve the internal string representation of the value for a variable.
-	const char* value() const throw ();
+    /// @returns true if the variable is defined, else false
+    bool isset() const throw () { return m_isset; }
 
-	/// @returns true if the variable is defined, else false
-	bool isset() const throw ();
 
 	/// Get the value as an integer
 	int enqi() const;
@@ -128,6 +140,9 @@ public:
 	/// Get the value as a string
 	const char* enqc() const;
 
+    /// Get the value as a std::string
+    std::string enqs() const;
+
 	/// Templated version of enq
 	template<typename T>
 	T enq() const
@@ -152,27 +167,34 @@ public:
 	/// Set the value from a double value
 	void setd(double val);
 
-	/// Set the value from a string value
-	void setc(const char* val);
+    /// Set the value from a string or opaque binary value
+    void setc(const char* val);
+
+    /// Set the value from a string or opaque binary value
+    void sets(const std::string& val);
+
+    /// Set from a value formatted with the format() method
+    void setf(const char* val);
 
     /**
-     * Set the raw, binary value from a string value.
-     *
-     * This is similar to setc(), but it always copies as many bytes as the
-     * variable is long, including null bytes.
-     */
-    void set_binary(const unsigned char* val);
-
-    /**
-     * Set the value from a string value, truncating \a val if it is too long
+     * Set the value from a string value, truncating it if it is too long.
      *
      * If a value is truncated, the last character is set to '>' to mark the
      * truncation.
      */
     void setc_truncate(const char* val);
 
-    /// Set from a value formatted with the format() method
-    void set_from_formatted(const char* val);
+    /**
+     * Set the value from another variable, performing conversions if
+     * needed. The attributes of \a src will be ignored.
+     */
+    void setval(const Var& src);
+
+    /**
+     * Replace all attributes in this variable with all the attributes from \a
+     * src
+     */
+    void setattrs(const Var& src);
 
 	/**
 	 * Shortcuts (use with care, as the semanthics are slightly different
@@ -183,7 +205,7 @@ public:
 	void set(double val) { setd(val); }
 	void set(const char* val) { setc(val); }
 	void set(const std::string& val) { setc(val.c_str()); }
-	void set(const Var& var) { copy_val(var); }
+	void set(const Var& var) { setval(var); setattrs(var); }
 	/// @}
 
 	/// Unset the value
@@ -203,12 +225,6 @@ public:
 	 */
 	const Var* enqa(Varcode code) const;
 
-    /**
-     * Query variable attribute according to significance given in CODE TABLE
-     * 031021
-     */
-    const Var* enqa_by_associated_field_significance(unsigned significance) const;
-
 	/**
 	 * Set an attribute of the variable.  An existing attribute with the same
 	 * wreport::Varcode will be replaced.
@@ -219,6 +235,16 @@ public:
 	 */
 	void seta(const Var& attr);
 
+    /**
+     * Set an attribute of the variable.  An existing attribute with the same
+     * wreport::Varcode will be replaced.
+     *
+     * @param attr
+     *   The attribute to add. Its value will be moved inside the destination
+     *   attribute, and attr will be unset.
+     */
+    void seta(Var&& attr);
+
 	/**
 	 * Set an attribute of the variable.  An existing attribute with the same
 	 * wreport::Varcode will be replaced.
@@ -227,7 +253,7 @@ public:
 	 *   The attribute to add.  It will be used directly, and var will take care of
 	 *   its memory management.
 	 */
-	void seta(std::auto_ptr<Var> attr);
+	void seta(std::unique_ptr<Var>&& attr);
 
 	/// Remove the attribute with the given code
 	void unseta(Varcode code);
@@ -242,46 +268,16 @@ public:
 	 */
 	const Var* next_attr() const;
 
-	/**
-	 * Set the value from another variable, performing conversions if
-	 * needed.
-	 *
-	 * The attributes of \a src will also be copied
-	 */
-	void copy_val(const Var& src);
-
     /**
-     * Set the value from another variable, performing conversions if
-     * needed.
+     * Create a formatted string representation of the variable value
      *
-     * The attributes of \a src will NOT be copied
+     * @param ifundef
+     *   String to use if the variable is undefiend
      */
-    void copy_val_only(const Var& src);
+    std::string format(const char* ifundef="") const;
 
-	/**
-	 * Copy all the attributes from another variable
-	 *
-	 * @param src
-	 *   The variable with the attributes to copy.
-	 */
-	void copy_attrs(const Var& src);
-
-    /**
-     * Copy all the attributes from another variable, unless they are set to an
-     * undefined value
-     *
-     * @param src
-     *   The variable with the attributes to copy.
-     */
-    void copy_attrs_if_defined(const Var& src);
-
-	/**
-	 * Create a formatted string representation of the variable value
-	 *
-	 * @param ifundef
-	 *   String to use if the variable is undefiend
-	 */
-	std::string format(const char* ifundef = "(undef)") const;
+    /// Write the formatted value of this variable to an output stream
+    void format(FILE* out, const char* ifundef="") const;
 
 	/**
 	 * Print the variable to an output stream
@@ -299,13 +295,13 @@ public:
      */
     void print(std::ostream& out) const;
 
-	/**
-	 * Print the variable to an output stream, without its attributes
-	 *
-	 * @param out
-	 *   The output stream to use for printing
-	 */
-	void print_without_attrs(FILE* out) const;
+    /**
+     * Print the variable to an output stream, without its attributes
+     *
+     * @param out
+     *   The output stream to use for printing
+     */
+    void print_without_attrs(FILE* out, const char* end="\n") const;
 
     /**
      * Print the variable to an output stream, without its attributes
@@ -346,10 +342,7 @@ template<> inline int Var::enq() const { return enqi(); }
 template<> inline float Var::enq() const { return (float)enqd(); }
 template<> inline double Var::enq() const { return enqd(); }
 template<> inline const char* Var::enq() const { return enqc(); }
-template<> inline std::string Var::enq() const { return enqc(); }
-
+template<> inline std::string Var::enq() const { return enqs(); }
 
 }
-
 #endif
-/* vim:set ts=4 sw=4: */
